#
# Top-Level Rule
#

.PHONY: all

ifeq "AGGREGATE" "$(PROJTYPE)"
all:
else
all: build
endif

#
# Local site-wide Makefile customization
#

ifndef LOCAL_MAKEFILEDIR
        LOCAL_MAKEFILEDIR = $(NEXT_ROOT)/LocalDeveloper/Makefiles/pb_makefiles
endif

-include $(LOCAL_MAKEFILEDIR)/common.make.preamble

#
# merging OS-specific variables
#

ifndef INSTALLDIR
ifeq "SOLARIS" "$(OS)"
INSTALLDIR = $(PDO_UNIX_INSTALLDIR)
else
ifeq "HPUX" "$(OS)"
INSTALLDIR = $(PDO_UNIX_INSTALLDIR)
else
INSTALLDIR = $($(OS)_INSTALLDIR)
endif
endif
endif

#
# Miscellaneous variables
#

SILENT = @
ARCH = $(shell $(ARCH_CMD))
ifndef RC_ARCHS
RC_ARCHS = $(ARCH)
endif
TARGET_ARCHS = $(RC_ARCHS)
MAKEFILE_SOURCE = pb_makefiles

ifeq "WINDOWS" "$(OS)"

LINK_SUBPROJECTS = NO
DISABLE_OBJCPLUSPLUS = NO
EXECUTABLE_EXT = .exe
ifeq "STATIC" "$(LIBRARY_STYLE)"
LIBRARY_EXT = .lib
else
LIBRARY_EXT = .dll
endif
LIBRARY_PREFIX =
DLL_EXT = .dll
NULL = NUL
ifneq "$(LIBRARY_STYLE)" "STATIC"
WINDOWS_DEF_FILE = $(NAME).def 
endif

else

LINK_SUBPROJECTS = YES
EXECUTABLE_EXT = 
LIBRARY_PREFIX = lib

ifeq "SOLARIS" "$(OS)"
ifeq "STATIC" "$(LIBRARY_STYLE)"
LIBRARY_EXT = .a
else
LIBRARY_EXT = .so
endif
else
ifeq "STATIC" "$(LIBRARY_STYLE)"
LIBRARY_EXT = .a
else
LIBRARY_EXT = .dylib
endif
endif

DLL_EXT =
NULL = /dev/null

endif

ifneq "NO" "$(INCLUDE_COMPATIBILITY_MAKEFILE)"
include $(MAKEFILEDIR)/compatibility.make
endif

#
# Roots
#

SRCROOT := $(shell pwd)
# BUILD_OUTPUT_DIR is defined below under Directories
ifndef OBJROOT
OBJROOT = $(BUILD_OUTPUT_DIR)
endif
ifndef SYMROOT
SYMROOT = $(BUILD_OUTPUT_DIR)
endif


#
# Versions
#

ifeq "" "$(COMPATIBILITY_PROJECT_VERSION)"
COMPATIBILITY_PROJECT_VERSION = 1
endif
ifeq "" "$(CURRENT_PROJECT_VERSION)"
CURRENT_PROJECT_VERSION = $(word 2, $(subst -, ,$(notdir $(SRCROOT))))
endif
ifeq "" "$(CURRENT_PROJECT_VERSION)"
CURRENT_PROJECT_VERSION = 1
endif

#
# Directories
#

ifneq "" "$($(OS)_BUILD_OUTPUT_DIR)"
BUILD_OUTPUT_DIR = $($(OS)_BUILD_OUTPUT_DIR)
endif
ifeq "" "$(BUILD_OUTPUT_DIR)"
BUILD_OUTPUT_DIR = $(SRCROOT)
endif

PRODUCT_DIR = $(SYMROOT)
OFILE_DIR = $(OBJROOT)/obj$(OFILE_DIR_SUFFIX)
SFILE_DIR = $(SYMROOT)/derived_src
PROJECT_HDR_DIR = $(SYMROOT)/ProjectHeaders
ifneq "" "$(PUBLIC_HDR_DIR)"
   PUBLIC_HDR_DIR = $(SYMROOT)/Headers/$(notdir $(PUBLIC_HDR_INSTALLDIR))
endif
ifeq "YES" "$(WRAPPED)"
   PUBLIC_HDR_DIR = $(PRODUCT)/Headers
endif
ifneq "" "$(PRIVATE_HDR_DIR)"
   PRIVATE_HDR_DIR = $(SYMROOT)/PrivateHeaders/$(notdir $(PRIVATE_HDR_INSTALLDIR))
endif
ifeq "YES" "$(WRAPPED)"
   PRIVATE_HDR_DIR = $(PRODUCT)/PrivateHeaders
endif
export PUBLIC_HDR_DIR
export PRIVATE_HDR_DIR
export PUBLIC_HDR_INSTALLDIR
export PRIVATE_HDR_INSTALLDIR

#
# File Lists
#

VPATH = $(OFILE_DIR) $(SFILE_DIR) $(LANGUAGE).lproj
GENERATED_SRCFILES = $(MSGFILES:%.msg=%Speaker.h) $(MSGFILES:%.msg=%Speaker.m) $(MSGFILES:%.msg=%Listener.h) $(MSGFILES:%.msg=%Listener.m) $(DEFSFILES:%.defs=%.h) $(DEFSFILES:%.defs=%User.c) $(DEFSFILES:%.defs=%Server.c) $(MIGFILES:%.mig=%.h) $(MIGFILES:%.mig=%User.c) $(MIGFILES:%.mig=%Server.c) $(PSWFILES:.psw=.h) $(PSWFILES:.psw=.c) $(PSWMFILES:.pswm=.h) $(PSWMFILES:.pswm=.m) $(subst .x_svc,.c, $(subst .x_clnt,.c, $(subst .x_xdr,.c, $(subst .x,.h, $(RPCFILES))))) $(YFILES:.y=.c) $(YFILES:.y=.h) $(LFILES:.l=.c) $(OTHER_GENERATED_SRCFILES)
SRCFILES = PB.project $(OTHERSRCS) $(HFILES) $(CLASSES) $(MFILES) $(CFILES) $(CAPCFILES) $(CAPMFILES) $(YFILES) $(LFILES) $(PSWFILES) $(PSWMFILES) $(LOCAL_RESOURCES) $(GLOBAL_RESOURCES) $(HELP_FILES) $(OTHERLINKED) $(OTHER_SOURCEFILES)

ifeq "YES" "$(LINK_SUBPROJECTS)"

SUBPROJ_OFILES = $(addsuffix _subproj.o,$(SUBPROJECTS))
OFILES = $(CLASSES:.m=.o) $(MFILES:.m=.o) $(CFILES:.c=.o) $(CAPCFILES:.C=.o) $(CAPMFILES:.M=.o) $(PSWFILES:.psw=.o) $(PSWMFILES:.pswm=.o) $(PROJTYPE_OFILES) $(OTHERLINKEDOFILES) $(OTHER_OFILES) $(OTHER_GENERATED_OFILES) $(SUBPROJ_OFILES)
OFILELISTS =

else

OFILES = $(CLASSES:.m=.o) $(MFILES:.m=.o) $(CFILES:.c=.o) $(CAPCFILES:.C=.o) $(CAPMFILES:.M=.o) $(PSWFILES:.psw=.o) $(PSWMFILES:.pswm=.o) $(PROJTYPE_OFILES) $(OTHERLINKEDOFILES) $(OTHER_OFILES) $(OTHER_GENERATED_OFILES)
ifneq "" "$(SUBPROJECTS)"
OFILELISTS = force_ofileList $(OFILE_DIR)/$(NAME).ofileList
.PHONY: force_ofileList
endif

endif


ALL_PRECOMPS = $(PRECOMPILED_HEADERS:.h=.p) $(PRECOMPS)
PRECOMPILED_PUBLIC_HEADERS = $(foreach X, $(PRECOMPILED_HEADERS), $(findstring $(X), $(PUBLIC_HEADERS)))
PRECOMPILED_PRIVATE_HEADERS = $(foreach X, $(PRECOMPILED_HEADERS), $(findstring $(X), $(PRIVATE_HEADERS)))
PRECOMPILED_PROJECT_HEADERS = $(foreach X, $(PRECOMPILED_HEADERS), $(findstring $(X), $(PROJECT_HEADERS)))

SUBPROJ_OFILELISTS = $(addsuffix _subproj.ofileList,$(SUBPROJECTS))
$(OFILELISTS) : $(SUBPROJ_OFILELISTS) 
	$(OFILE_LIST_TOOL) -o $(OFILE_DIR)/$(NAME).ofileList $(SUBPROJ_OFILELISTS)

DEPENDENCIES = $(SRCFILES) $(OFILES) $(OFILELISTS) $(OTHER_PRODUCT_DEPENDS)
LOADABLES = $(OFILES) $(OFILELISTS:%=-filelist %) $(LIBS) $(FRAMEWORKS) $(PROJTYPE_LIBS) $(PROJTYPE_FRAMEWORKS) $(OTHER_LIBS) $(OTHER_FRAMEWORKS) $(OPTIONAL_LIBS) $(OPTIONAL_FRAMEWORKS)
GARBAGE = $(OBJROOT)/obj $(OBJROOT)/obj-* $(SFILE_DIR) $(PROJECT_HDR_DIR) $(PROJTYPE_GARBAGE) $(OTHER_GARBAGE) $(PRECOMPILED_HEADERS:.h=.p) $(PRECOMPS)


include $(MAKEFILEDIR)/versions.make
include $(MAKEFILEDIR)/commands-$(OS).make
include $(MAKEFILEDIR)/flags.make
include $(MAKEFILEDIR)/implicitrules.make
include $(MAKEFILEDIR)/recursion.make

# The following can and should be overridden by local customizations
include $(MAKEFILEDIR)/sgs.make

ifneq "AGGREGATE" "$(PROJTYPE)"
include $(MAKEFILEDIR)/prebuild.make
include $(MAKEFILEDIR)/build.make
include $(MAKEFILEDIR)/installhdrs.make
include $(MAKEFILEDIR)/install.make
endif
include $(MAKEFILEDIR)/installsrc.make

always:


#
# .def-file generation (used on Windows only)
#
ifeq "WINDOWS" "$(OS)"
TMPFILE = $(OFILE_DIR)/nmtmpfile
GENERATED_DEF_FILE = $(SFILE_DIR)/$(WINDOWS_DEF_FILE)

$(WINDOWS_DEF_FILE): $(OFILES) $(OTHER_OFILES)
	$(SILENT) if [ ! -r "./$(WINDOWS_DEF_FILE)" ] ; then \
		$(ECHO) -n "Generating $(notdir $@)...." ; \
		(cd $(OFILE_DIR); $(DUMP_SYMBOLS) $(OFILES) $(OTHER_OFILES) `$(CAT) $(NAME).ofileList` | $(EGREP) "(SECT).*(External).*(\|)" | $(EGREP) -v "(__GLOBAL_$I)|(_OBJC_)" | $(SED) "s/ _/ /" > $(TMPFILE)) ; \
		$(ECHO) "LIBRARY $(NAME).dll" > $(GENERATED_DEF_FILE) ; \
		$(ECHO) "EXPORTS" >> $(GENERATED_DEF_FILE) ; \
		$(CAT) $(TMPFILE) | $(EGREP) "(SECT2)|(.objc_c)" | $(AWK) '{printf "\t%s CONSTANT\n", $$NF}' >> $(GENERATED_DEF_FILE) ; \
		$(CAT) $(TMPFILE) | $(EGREP) -v "(SECT2)|(.objc_c)" | $(AWK) '{printf "\t%s\n", $NF}' >> $(GENERATED_DEF_FILE) ; \
		$(RM) $(TMPFILE) ; \
		$(ECHO) "done"; \
	fi
endif

clean_deffile:
	$(RM) -f $(WINDOWS_DEF_FILE)


#
# Sequencing
#

ifneq "YES" "$(RECURSING)"
announce-prebuild announce-build announce-installhdrs announce-install: display-masthead
endif

ifneq "AGGREGATE" "$(PROJTYPE)"
ifneq "YES" "$(RECURSING)"
ifneq "YES" "$(SKIP_BUILD)"
announce-install: build
endif
ifneq "YES" "$(SKIP_PREBUILD)"
announce-build: prebuild
endif
endif
endif

#
# Conveniences
#

profile:
	$(MAKE) PROFILE=YES APP_WRAPPER_EXTENSION=.profile

debug:
	$(MAKE) DEBUG=YES PROFILE=NO OPTIMIZE=NO APP_WRAPPER_EXTENSION=.debug

#
# Masthead
#

display-masthead:
	$(SILENT) $(ECHO) == Making $(NAME) for $(TARGET_ARCHS) ==

#
# Makefile-Debugging Rules
#

.PHONY: show-variable sv show-expression se

VARIABLE=$(V)
show-variables sv:
	$(SILENT) $(foreach X, $(VARIABLE), $(ECHO) "$($(X))";)
EXPRESSION=$(E)
show-expression se:
	$(SILENT) $(foreach X, $(EXPRESSION), $(ECHO) "$(X)";)

#
# Cleaning Rules
#

.PHONY: clean mostlyclean announce-clean

clean: announce-clean
	$(RM) -rf $(GARBAGE) $(PRODUCTS)

mostlyclean: announce-clean
	$(RM) -rf $(GARBAGE)
	
announce-clean:
	$(SILENT) $(ECHO) == Cleaning $(NAME) ==


-include $(LOCAL_MAKEFILEDIR)/common.make.postamble
